@@ -20,26 +20,28 @@
 
 import logging
 import os
-from past.builtins import basestring
 import copy
 
 import numpy as np
 from PIL import Image
-from qtpy import QtCore
+import h5py
 
 import fabio
 
-from .util.spe import SpeFile
+from .util import Signal
+from dioptas.model.loader.spe import SpeFile
 from .util.NewFileWatcher import NewFileInDirectoryWatcher
 from .util.HelperModule import rotate_matrix_p90, rotate_matrix_m90, FileNameIterator
 from .util.ImgCorrection import ImgCorrectionManager, ImgCorrectionInterface, TransferFunctionCorrection
-from .util.LambdaLoader import LambdaImage
-from .util.KaraboLoader import KaraboFile
+from dioptas.model.loader.LambdaLoader import LambdaImage
+from dioptas.model.loader.KaraboLoader import KaraboFile
+from dioptas.model.loader.hdf5Loader import Hdf5Image
+from dioptas.model.loader.FabioLoader import FabioLoader
 
 logger = logging.getLogger(__name__)
 
 
-class ImgModel(QtCore.QObject):
+class ImgModel(object):
     """
     Main Image handling class. Supports several features:
         - loading image files in any format using fabio
@@ -51,10 +53,6 @@ class ImgModel(QtCore.QObject):
     In order to subscribe to changes of the data in the ImgModel, please use the img_changed QtSignal.
     The Signal will be called every time the img_data has changed.
     """
-    img_changed = QtCore.Signal()
-    autoprocess_changed = QtCore.Signal()
-    transformations_changed = QtCore.Signal()
-    corrections_removed = QtCore.Signal()
 
     def __init__(self):
         super(ImgModel, self).__init__()
@@ -66,6 +64,7 @@ class ImgModel(QtCore.QObject):
 
         self.series_pos = 1
         self.series_max = 1
+        self.selected_source = None
 
         self._img_data = None
         self._img_data_background_subtracted = None
@@ -97,7 +96,14 @@ class ImgModel(QtCore.QObject):
 
             # function to get an image in the current series. A function assigned to this attribute should take
             # a single parameter pos (position in the series starting at 0) and return a 2d array with the image data
-            {"name": "series_get_image", "default": None, "attribute": "series_get_image"}
+            {"name": "series_get_image", "default": None, "attribute": "series_get_image"},
+
+            # list of sources for different image series within 1 file. This is used by an HDF5 file with several
+            # datasets
+            {"name": "sources", "default": None, "attribute": "sources"},
+
+            # a function to select a source:
+            {"name": "select_source", "default": None, "attribute": "_select_source"}
         ]
 
         # set the loadable attributes to their defaults
@@ -108,24 +114,31 @@ class ImgModel(QtCore.QObject):
         # setting up autoprocess
         self._autoprocess = False
         self._directory_watcher = NewFileInDirectoryWatcher(
-            file_types=['.img', '.sfrm', '.dm3', '.edf', '.xml',
-                        '.cbf', '.kccd', '.msk', '.spr', '.tif',
-                        '.mccd', '.mar3450', '.pnm', 'spe']
+            file_types=['img', 'sfrm', 'dm3', 'edf', 'xml',
+                        'cbf', 'kccd', 'msk', 'spr', 'tif',
+                        'mccd', 'mar3450', 'pnm', 'spe']
         )
         self._directory_watcher.file_added.connect(self.load)
 
-    def load(self, filename):
+        # define the signals
+        self.img_changed = Signal()
+        self.autoprocess_changed = Signal()
+        self.transformations_changed = Signal()
+        self.corrections_removed = Signal()
+
+    def load(self, filename, pos=0):
         """
         Loads an image file in any format known by fabIO, PIL or HDF5. Automatically performs all previous img
         transformations, recalculates background subtracted and absorption corrected image data.
         The img_changed signal will be emitted after the process.
         :param filename: path of the image file to be loaded
+        :param pos: position of image in the image file to be loaded
         """
         filename = str(filename)  # since it could also be QString
         logger.info("Loading {0}.".format(filename))
         self.filename = filename
 
-        image_file_data = self.get_image_data(filename)
+        image_file_data = self.get_image_data(filename, pos)
         self.set_loadable_attributes(image_file_data)
 
         self.file_name_iterator.update_filename(filename)
@@ -133,25 +146,28 @@ class ImgModel(QtCore.QObject):
 
         self._perform_img_transformations()
         self._calculate_img_data()
+        self.series_pos = pos + 1
 
         self.img_changed.emit()
 
-    def get_image_data(self, filename):
+    def get_image_data(self, filename, pos=0):
         """
         Tries to load the given file using different image loader libraries and returns a dictionary containing all
         retrieved file data.
         :param filename: string containing a path to an image file
+        :param pos: position of image in the image file to be loaded
         :return: dictionary containing all retrieved file information. Look at "loadable data" for possible key names.
                  Present key names depend on applied image loader
         """
-        img_loaders = [self.load_PIL, self.load_spe, self.load_fabio, self.load_lambda, self.load_karabo]
+        img_loaders = [self.load_PIL, self.load_spe, self.load_fabio, self.load_lambda, self.load_karabo,
+                       self.load_hdf5]
 
         for loader in img_loaders:
-            data = loader(filename)
+            data = loader(filename, pos)
             if data:
                 return data
         else:
-            raise IOError("No handler found for given image")
+            raise IOError("No handler found for given image with filename: " + filename)
 
     def set_loadable_attributes(self, loaded_data):
         """
@@ -167,7 +183,7 @@ class ImgModel(QtCore.QObject):
             else:
                 self.__setattr__(attribute["attribute"], copy.copy(attribute["default"]))
 
-    def load_PIL(self, filename):
+    def load_PIL(self, filename, *args):
         """
         Loads an image using the PIL library. Also returns file and motor info if present
         :param filename: path to the image file to be loaded
@@ -191,7 +207,7 @@ class ImgModel(QtCore.QObject):
         except IOError:
             return None
 
-    def load_spe(self, filename):
+    def load_spe(self, filename, *args):
         """
         Loads an image using the builtin spe library.
         :param filename: path to the image file to be loaded
@@ -203,23 +219,29 @@ class ImgModel(QtCore.QObject):
         else:
             return None
 
-    def load_fabio(self, filename):
+    def load_fabio(self, filename, frame_index=0):
         """
         Loads an image using the fabio library.
         :param filename: path to the image file to be loaded
+        :param frame_index: frame index of the image file to be loaded inside of multi-frame file
         :return: dictionary with image_data and image_data_fabio, None if unsuccessful
         """
         try:
-            img_data_fabio = fabio.open(filename)
-            img_data = img_data_fabio.data[::-1]
-            return {"img_data_fabio": img_data_fabio, "img_data": img_data}
+            self.loader = FabioLoader(filename)
+            return {
+                "img_data_fabio": self.loader.fabio_image,
+                "img_data": self.loader.get_image(frame_index),
+                "series_max": self.loader.series_max,
+                "series_get_image": self.loader.get_image
+            }
         except (IOError, fabio.fabioutils.NotGoodReader):
             return None
 
-    def load_lambda(self, filename):
+    def load_lambda(self, filename, frame_index=0):
         """
         loads an image made by a lambda detector using the builtin lambda library.
         :param filename: path to the image file to be loaded
+        :param frame_index: frame index of the image file to be loaded inside of multi-frame file
         :return: dictionary with img_data, series_max and series_get_image, None if unsuccessful
         """
         try:
@@ -227,14 +249,17 @@ class ImgModel(QtCore.QObject):
         except IOError:
             return None
 
-        return {"img_data": lambda_im.get_image(0),
+        if frame_index >= lambda_im.series_max:
+            return None
+        return {"img_data": lambda_im.get_image(frame_index),
                 "series_max": lambda_im.series_max,
                 "series_get_image": lambda_im.get_image}
 
-    def load_karabo(self, filename):
+    def load_karabo(self, filename, frame_index=0):
         """
         Loads an Imageseries created from within the karabo-framework at XFEL.
         :param filename: path to the *.h5 karabo file
+        :param frame_index: position of image in the image file to be loaded
         :return: dictionary with img_data of the first train_id, series_start, series_max and series_get_image,
                  None if unsuccessful
         """
@@ -242,11 +267,48 @@ class ImgModel(QtCore.QObject):
             karabo_file = KaraboFile(filename)
         except IOError:
             return None
-
-        return {"img_data": karabo_file.get_image(0),
+        if frame_index >= karabo_file.series_max:
+            return None
+        return {"img_data": karabo_file.get_image(frame_index),
                 "series_max": karabo_file.series_max,
                 "series_get_image": karabo_file.get_image}
 
+    def load_hdf5(self, filename, frame_index=0):
+        """
+        Loads an ESRF hdf5 file
+        :param filename: filename with path to *.h5 ESRF file
+        :param frame_index: frame index for multi-image file
+        :return: dictionary with img_data of the first image in the first source, dataset_list, series_max, and
+                 series_get_image
+        """
+
+        hdf5_image = Hdf5Image(filename)
+        self.loader = hdf5_image
+        self.selected_source = hdf5_image.image_sources[0]
+
+        return {"img_data": hdf5_image.get_image(frame_index),
+                "series_max": hdf5_image.series_max,
+                "series_get_image": hdf5_image.get_image,
+                "sources": hdf5_image.image_sources,
+                "select_source": hdf5_image.select_source
+                }
+
+    def select_source(self, source):
+        """
+        Selects a source from the available sources and loads updates the current image in the model.
+        :param source: string for source (check sources for available strings for the corresponding file)
+        """
+        self._select_source(source)
+        self.selected_source = source
+        self.series_max = self.loader.series_max
+        self.series_pos = min(self.series_pos, self.series_max)
+        self._img_data = self.series_get_image(self.series_pos - 1)
+
+        self._perform_img_transformations()
+        self._calculate_img_data()
+
+        self.img_changed.emit()
+
     def save(self, filename):
         """
         Saves the current file as another image file, the raw data is used for saving.
@@ -487,6 +549,9 @@ class ImgModel(QtCore.QObject):
             background subtraction. in case you want the raw data without corrections, please use the
             raw_img_data property.
         """
+        # if self._img_data is None:
+        #     return None
+
         if self._background_data is None and not self._img_corrections.has_items():
             return self._img_data * self.factor
 
@@ -653,7 +718,7 @@ class ImgModel(QtCore.QObject):
         :param correction: An Object inheriting the ImgCorrectionInterface.
         :type correction: ImgCorrectionInterface
         :param name: correction can be given a name, to selectively delete or obtain later.
-        :type name: basestring
+        :type name: str
         """
         self._img_corrections.add(correction, name)
         self._calculate_img_data()
@@ -702,6 +767,7 @@ class ImgModel(QtCore.QObject):
         reads the file info from tif_tags and returns a file info
         """
         result = ""
+        end_result = ""
         tags = image.tag
         useful_keys = []
         for key in tags.keys():
@@ -711,11 +777,14 @@ class ImgModel(QtCore.QObject):
         useful_keys.sort()
         for key in useful_keys:
             tag = tags[key][0]
-            if isinstance(tag, basestring):
+            if isinstance(tag, str):
                 new_line = str(tag) + "\n"
                 new_line = new_line.replace(":", ":\t", 1)
-                result += new_line
-        return result
+                if 'TIFFImageDescription' in new_line:
+                    end_result = new_line
+                else:
+                    result += new_line
+        return result + end_result
 
     def _get_motors_info(self, image):
         """
@@ -759,6 +828,12 @@ class ImgModel(QtCore.QObject):
         self._factor = new_value
         self.img_changed.emit()
 
+    def blockSignals(self, block=True):
+        for member in vars(self):
+            attr = getattr(self, member)
+            if isinstance(attr, Signal):
+                attr.blocked = block
+
 
 class BackgroundDimensionWrongException(Exception):
     pass
